--[[

ElvUI Location Plus
an addon for ElvUI, that adds player location and coords + 2 Datatexts
by Benik @ Emerald Dream EU
----------------------------------------------------------------------------------
   Hints: 
a. Click to Toggle WorldMap
b. ShiftClick to send your location in chat
c. CtrlClick to toggle left/right datatext panels
----------------------------------------------------------------------------------
   Credits:
- Elv, Blazeflack, Darth Predator, Talryn, for showing me the best way to do this
- Phenox, Blacknet, CohesioN`(idea from his TukUI edit)
- CodeRedLin, iceeagle, daf2(from Broker_Location addon)
for digging their great code and making this possible.
- Tuk, for making BlizzUI playable and Elv's forums community.
----------------------------------------------------------------------------------
	ToDo:
-- Add font options
-- Less reload ui when changing layout
]]--

local E, L, DF = unpack(ElvUI); --Inport: Engine, Locales, ProfileDB, GlobalDB
local DT = E:GetModule('DataTexts')
local M = E:GetModule('Minimap')

local tourist = LibStub("LibTourist-3.0")

local LOCPANEL_HEIGHT = 22 --all panels height
local LOCPANEL_WIDTH = 200 --Location panel width, though its size depends on the location string
local COORDS_WIDTH = 30    --Coord panels width
local DTPANELS_WIDTH = 100 --Datatext panels width

local ElvUI_Location = CreateFrame('Frame', 'ElvUI_Location')
ElvUI_Location.version = '1.07'


-- mouse over the location panel
local function LocPanel_OnEnter(self,...)
	if loc_db.loc_shadow then
		GameTooltip:SetOwner(self, "ANCHOR_BOTTOM", 0, -3); -- new tooltip position when shadows enabled
	else
		GameTooltip:SetOwner(self, "ANCHOR_BOTTOM", 0, -1); -- default tooltip position
	end
	GameTooltip:ClearAllPoints()
	GameTooltip:SetPoint("BOTTOM", self, "BOTTOM", 0, 0)
	GameTooltip:ClearLines()
	GameTooltip:AddDoubleLine(L["Zone : "], GetRealZoneText(),1,1,1,selectioncolor)
	GameTooltip:AddDoubleLine(L["Continent : "], tourist:GetContinent(GetRealZoneText()),1,1,1,selectioncolor)
	
	if loc_db.loc_tt then
		if loc_db.loc_tthint then
			GameTooltip:AddLine(" ")
			GameTooltip:AddDoubleLine(L["Click : "], L["Toggle WorldMap"],0.7,0.7,1,0.7,0.7,1)
			GameTooltip:AddDoubleLine(L["ShiftClick : "], L["Send position to chat"],0.7,0.7,1,0.7,0.7,1)
			GameTooltip:AddDoubleLine(L["CtrlClick : "], L["Toggle Datatexts"],0.7,0.7,1,0.7,0.7,1)
		end
		GameTooltip:Show()
		else
		GameTooltip:Hide()
	end
end

-- mouse leaving the location panel
local function LocPanel_OnLeave(self,...)
	GameTooltip:Hide()
end

-- clicking the location panel
local function LocPanel_OnClick(self,...)
	if IsShiftKeyDown() then
		local edit_box = _G.ChatEdit_ChooseBoxForSend()
		local x, y = GetPlayerMapPosition("player")
		local message
		local coords = string.format("%.0f, %.0f", x * 100, y * 100)
			if GetRealZoneText() ~= GetSubZoneText() then
				message = string.format("%s: %s (%s)", GetRealZoneText(), GetSubZoneText(), coords)
			else
				message = string.format("%s (%s)", GetRealZoneText(), coords)
			end
		_G.ChatEdit_ActivateChat(edit_box)
		edit_box:Insert(message) 
	else
		if IsControlKeyDown() then
			ToggleFrame(LeftCoordDtPanel)
			ToggleFrame(RightCoordDtPanel)
		else 
			ToggleFrame(WorldMapFrame)
		end
	end
end

-- Location panel
function CreateMainLocPanel()
	local loc_panel = CreateFrame('Button', 'LocationPlusPanel', E.UIParent)
	
	if loc_db.loc_trans == false then
		loc_panel:SetTemplate('Default', true)	
	else
		loc_panel:SetTemplate('Transparent')
	end
	if loc_db.loc_shadow then
		loc_panel:CreateShadow('Default')
	end

	loc_panel:Width(LOCPANEL_WIDTH)
	
	if loc_db.loc_ht then
		loc_panel:Height((LOCPANEL_HEIGHT)+6)  -- height option on Location Panel	
	else
		loc_panel:Height(LOCPANEL_HEIGHT) -- default height
	end

	loc_panel:Point('TOP', E.UIParent, 'TOP', 0, -E.mult-30)
	loc_panel:SetFrameStrata('LOW')
	loc_panel:RegisterForClicks('AnyUp')
	loc_panel:SetScript('OnEnter', LocPanel_OnEnter) 
	loc_panel:SetScript('OnLeave', LocPanel_OnLeave)
	loc_panel:SetScript('OnClick', LocPanel_OnClick)

	-- Location Text
	local Text = LocationPlusPanel:CreateFontString(nil, "LOW")
	Text:FontTemplate(E["media"].dtFont)
	Text:Point("CENTER", 0, 0)
	
	loc_panel:SetScript("OnUpdate", function(self,event,...)
		local subZoneText = GetMinimapZoneText() or ""
		local zoneText = GetRealZoneText()
		local displayLine	
		-- zone and subzone
		if loc_db.loc_both then
			if (subZoneText ~= "") and (subZoneText ~= zoneText) then
				displayLine = zoneText .. ": " .. subZoneText
			else
				displayLine = subZoneText
			end
		else
			displayLine = subZoneText
		end

		Text:SetTextColor(M:GetLocTextColor())
		Text:SetText(displayLine)
		LocationPlusPanel:SetWidth(Text:GetWidth() + 24)
	end)
	
	--Hide in combat
	loc_panel:SetScript("OnEvent",function(self, event)
		if loc_db.loc_combat then
			if event == "PLAYER_REGEN_DISABLED" then
				UIFrameFadeOut(LocationPlusPanel, 0.2, LocationPlusPanel:GetAlpha(), 0)
				LocationPlusPanel.fadeInfo.finishedFunc = OnFade
			else
				loc_panel:Show()	
			end
			if event == "PLAYER_REGEN_ENABLED" then
				UIFrameFadeIn(LocationPlusPanel, 0.2, LocationPlusPanel:GetAlpha(), 1)
				LocationPlusPanel:Show()
			end
		end
	end)

	-- Mover
	E:CreateMover(LocationPlusPanel, "LocationMover", "Location Panel")
end

local function OnFade()
	LocationPlusPanel:Hide()
	LeftCoordDtPanel:Hide()
end

-- Coord panels
function CreateLocCoordPanels()

	-- X Coord panel
	local coordsX = CreateFrame('Frame', "XCoordsPanel", LocationPlusPanel)

	if loc_db.loc_trans == false then
		coordsX:SetTemplate('Default', true)
	else
		coordsX:SetTemplate('Transparent')
	end
	coordsX:Width(COORDS_WIDTH)
	coordsX:Height(LOCPANEL_HEIGHT)
	if loc_db.loc_shadow == false then
		coordsX:Point('RIGHT', LocationPlusPanel, 'LEFT', -1, 0) -- default XCoords position
		coordsX:Point('CENTER', LocationPlusPanel, 'CENTER', -1, 0) -- default XCoords position
	else
		coordsX:CreateShadow('Default') -- shadows on X
		coordsX:Point('RIGHT', LocationPlusPanel, 'LEFT', -3, 0) -- new XCoords position when shadows enabled
		coordsX:Point('CENTER', LocationPlusPanel, 'CENTER', -3, 0) -- new XCoords position when shadows enabled
	end
	coordsX:SetFrameStrata(LocationPlusPanel:GetFrameStrata())

	-- Y Coord panel
	local coordsY = CreateFrame('Frame', "YCoordsPanel", LocationPlusPanel)

	if loc_db.loc_trans == false then
		coordsY:SetTemplate('Default', true)
	else
		coordsY:SetTemplate('Transparent')
	end
	coordsY:Width(COORDS_WIDTH)
	coordsY:Height(LOCPANEL_HEIGHT)
	if loc_db.loc_shadow then
		coordsY:CreateShadow('Default') -- shadows on Y
		coordsY:Point('LEFT', LocationPlusPanel, 'RIGHT', 3, 0) -- new YCoords position when shadows enabled
		coordsY:Point('CENTER', LocationPlusPanel, 'CENTER', 3, 0) -- new YCoords position when shadows enabled
	else
		coordsY:Point('LEFT', LocationPlusPanel, 'RIGHT', 1, 0) -- default YCoords position
		coordsY:Point('CENTER', LocationPlusPanel, 'CENTER', 1, 0) -- default YCoords position
	end
	coordsY:SetFrameStrata(LocationPlusPanel:GetFrameStrata())
	
	local TextX = XCoordsPanel:CreateFontString(nil, "LOW")
	TextX:FontTemplate()
	TextX:SetPoint("CENTER", 0, 0)

	local TextY = YCoordsPanel:CreateFontString(nil, "LOW")
	TextY:FontTemplate()
	TextY:SetPoint("CENTER", 0, 0)

	local ela,go = 0,false

	local cUpdate = function(self,t)
		ela = ela - t
		if ela > 0 then return end
			local x,y = GetPlayerMapPosition("player")
			local xt,yt
			x = math.floor(100 * x)
			y = math.floor(100 * y)
				if x == 0 and y == 0 then
					TextX:SetText("-")
					TextY:SetText("-")
				else
				if x < 10 then
					xt = "0"..x
				else
					xt = x
				end
			if y < 10 then
				yt = "0"..y
			else
				yt = y
			end
			TextX:SetText(xt)
			TextY:SetText(yt)
		end
	end
	ela = .2

	XCoordsPanel:SetScript("OnUpdate", cUpdate)
	SetMapToCurrentZone() -- gonna test this one if it helps on accurate coords

end

-- Datatext panels
function CreateLocDTPanels()

	-- Left coords Datatext panel
	if loc_db.loc_trans == false then
		left_dtp:SetTemplate('Default', true)
	else
		left_dtp:SetTemplate('Transparent')
	end
	left_dtp:Width(DTPANELS_WIDTH)
	left_dtp:Height(LOCPANEL_HEIGHT)
	
	if loc_db.loc_shadow then
		left_dtp:CreateShadow('Default') -- left dt shadows option
		left_dtp:Point('RIGHT', XCoordsPanel, 'LEFT', -3, 0) -- new left panel position when shadows enabled
		left_dtp:Point('CENTER', XCoordsPanel, 'CENTER', -3, 0) -- new left panel position when shadows enabled
	else
		left_dtp:Point('RIGHT', XCoordsPanel, 'LEFT', -1, 0) -- default left panel position
		left_dtp:Point('CENTER', XCoordsPanel, 'CENTER', -1, 0) -- default left panel position
	end
	
	left_dtp:SetFrameStrata('LOW')
	left_dtp:SetParent(LocationPlusPanel) -- set to hide in combat
	left_dtp:Show()

	-- Right coords Datatext panel
	if loc_db.loc_trans == false then
		right_dtp:SetTemplate('Default', true)
	else
		right_dtp:SetTemplate('Transparent')
	end
	right_dtp:Width(DTPANELS_WIDTH)
	right_dtp:Height(LOCPANEL_HEIGHT)
	
	if loc_db.loc_shadow then
		right_dtp:CreateShadow('Default') -- right dt shadows option
		right_dtp:Point('LEFT', YCoordsPanel, 'RIGHT', 3, 0) -- new right panel position when shadows enabled
		right_dtp:Point('CENTER', YCoordsPanel, 'CENTER', 3, 0) -- new right panel position when shadows enabled
	else
		right_dtp:Point('LEFT', YCoordsPanel, 'RIGHT', 1, 0) -- default right panel position
		right_dtp:Point('CENTER', YCoordsPanel, 'CENTER', 1, 0) -- default right panel position	
	end
	
	right_dtp:SetFrameStrata('LOW')
	right_dtp:SetParent(LocationPlusPanel) -- set to hide in combat
	right_dtp:Show()
	
end

do
	left_dtp = CreateFrame('Frame', 'LeftCoordDtPanel', E.UIParent)
	DT:RegisterPanel(LeftCoordDtPanel, 1, 'ANCHOR_BOTTOM', 0, -4)
	right_dtp = CreateFrame('Frame', 'RightCoordDtPanel', E.UIParent)
	DT:RegisterPanel(RightCoordDtPanel, 1, 'ANCHOR_BOTTOM', 0, -4)
end

-- How to appear in datatext options
L['RightCoordDtPanel'] = L["LocationPlus Right Panel"];
L['LeftCoordDtPanel'] = L["LocationPlus Left Panel"];

-- Setting default datatexts
DF.datatexts.panels.RightCoordDtPanel = 'System'
DF.datatexts.panels.LeftCoordDtPanel = 'Durability'

ElvUI_Location:RegisterEvent("PLAYER_ENTERING_WORLD")
ElvUI_Location:SetScript("OnEvent",function(self, event)
    if event == "PLAYER_ENTERING_WORLD" then
		if loc_db.loc_enable then
			CreateMainLocPanel()
			CreateLocCoordPanels()
			CreateLocDTPanels()
			self:RegisterEvent("ZONE_CHANGED_NEW_AREA")
			self:RegisterEvent("ZONE_CHANGED")
			self:RegisterEvent("ZONE_CHANGED_INDOORS")
			LocationPlusPanel:RegisterEvent("PLAYER_REGEN_DISABLED")
			LocationPlusPanel:RegisterEvent("PLAYER_REGEN_ENABLED")
			print(L["Location Plus "]..format("v|cff33ffff%s|r",ElvUI_Location.version)..L[" is loaded. Thank you for using it."])
		end
	end
end)

